﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.IO;
using System.IO.Compression;
using System.Xml;
using System.Windows.Forms;
using ICSharpCode.SharpZipLib.Zip;

namespace PCSX2_Profile_Manager
{
    /// <summary>
    /// This class is responsible of (mainly) making code cleaner and easier to read,
    /// but it also serves a purpose in the program, to get and set configuration values
    /// with ease and doing error-checking in the process.
    /// </summary>
    public static class Config
    {
        public static string PCSX2Path { get; set; }    /// PCSX2 path. Ex: C:\Games\PCSX2\
        public static string PCSX2Exe { get; set; }     /// EXE file. Ex: C:\Games\PCSX2\pcsx2-rXXXX.exe
        public static string INIsPath { get; set; }     /// INI files path. Ex: C:\Games\PCSX2\inis
        public static string ProfilesPath { get; set; } /// PPM's profile path. Ex: C:\Games\PCSX2\profiles
        public static int DCAction { get; set; }        /// Action to execute on frmMain listbox's double-click.

        /// <summary>
        /// This function should be self-explanatory. We load Config.dat if it exists, else we
        /// return a False value. We also read DCAction's value and set it. If a file named
        /// profile.dat doesn't exist in the INIsPath, we create a Default profile off any
        /// .ini files in INIsPath and save it.
        /// </summary>
        public static bool Load()
        {
            string ConfigFile = Application.StartupPath + @"\Config.dat";
            if (File.Exists(ConfigFile))
            {
                using (StreamReader r = new StreamReader(ConfigFile))
                {
                    PCSX2Exe = r.ReadLine();
                    PCSX2Path = Path.GetDirectoryName(PCSX2Exe);
                    INIsPath = PCSX2Path + @"\inis\";
                    ProfilesPath = INIsPath + @"\profiles\";
                    DCAction = int.Parse(r.ReadLine());
                }
                if (!File.Exists(INIsPath + "profile.dat"))
                {
                    Profile.Save("Default", true);
                    Profile.Activate("Default", true);
                }
                return true;
            }
            return false;
        }

        /// <summary>
        /// Save config, duh.
        /// </summary>
        public static bool Save()
        {
            string ConfigFile = Application.StartupPath + @"\Config.dat";
            using (StreamWriter w = new StreamWriter(ConfigFile))
            {
                w.WriteLine(PCSX2Exe);
                w.Write(DCAction);
            }
            return true;
        }
    }

    /// <summary>
    /// This class handles everything related to profiles. It also makes the code cleaner
    /// and easier to read.
    /// </summary>
    public static class Profile
    {
        /// <summary>
        /// Clear the current profile. It's the same as deleting all files in the inis folder.
        /// </summary>
        public static void Clear()
        {
            string[] filesToDelete = Directory.GetFiles(Config.INIsPath);
            foreach (string file in filesToDelete) { File.Delete(file); }
        }

        /// <summary>
        /// Save the current profile, taking all INI files found in the inis folder with it.
        /// </summary>
        /// <param name="name">Profile's name</param>
        /// <param name="silent">Show messages on success/error?</param>
        public static void Save(string name, bool silent)
        {
            string[] files = Directory.GetFiles(Config.INIsPath, "*.ini");
            Save(files, name, silent);
        }

        /// <summary>
        /// Same as above, but it allows to set which files to save. It is used along with frmSelectINIs.
        /// </summary>
        /// <param name="files">Files to save</param>
        /// <param name="name">Profile's name</param>
        /// <param name="silent">Show messages on success/error?</param>
        public static void Save(string[] files, string name, bool silent)
        {
            try
            {
                /// If the profiles path doesn't exist, we create it.
                if (!Directory.Exists(Config.ProfilesPath)) { Directory.CreateDirectory(Config.ProfilesPath); }

                /// Now it's time to create the ZIP file.
                using (ZipOutputStream s = new ZipOutputStream(File.Create(Config.ProfilesPath + name + ".zip")))
                {
                    /// 0 = Store files, no compression. We don't need it anyway, since they're all less than 1Mb INI files.
                    s.SetLevel(0);

                    /// Just an 1Mb read buffer, nothing fancy here. Could be less, but who cares :P
                    byte[] buffer = new byte[1024];

                    foreach (string file in files)
                    {
                        string fileName = String.Empty;

                        /// This is a dirty bugfix. Sometimes "file" would contain the filename with the INIs path already set. In that case
                        /// we don't need to append it again, thus, causing errors when compressing a non-existant file due to the incorrect
                        /// path set.
                        if (!file.Contains(Config.INIsPath)) { fileName = Config.INIsPath + file; } else { fileName = file; }
                        if (fileName != String.Empty)
                        {
                            /// The file exists, so we add it to the ZIP file.
                            ZipEntry entry = new ZipEntry(Path.GetFileName(file));
                            entry.DateTime = DateTime.Now;
                            s.PutNextEntry(entry);

                            /// Read the file's data and write it.
                            using (FileStream fs = File.OpenRead(fileName))
                            {
                                int sourceBytes;
                                do
                                {
                                    sourceBytes = fs.Read(buffer, 0, buffer.Length);
                                    s.Write(buffer, 0, sourceBytes);
                                } while (sourceBytes > 0);
                            }
                        }
                    }
                    s.Finish();
                    s.Close();
                }
                if (!silent) { MessageBox.Show("Profile saved", "Success!"); }
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message, "Error saving profile");
            }
        }

        /// <summary>
        /// Activate a profile
        /// </summary>
        /// <param name="name">The profile's name to activate</param>
        /// <param name="silent">Show messages on success/error?</param>
        public static void Activate(string name, bool silent)
        {
            /// We clear the current profile before extracting the INI files.
            Clear();
            if(File.Exists(Config.ProfilesPath + name + ".zip"))
            {
                using (ZipInputStream s = new ZipInputStream(File.OpenRead(Config.ProfilesPath + name + ".zip")))
                {
                    ZipEntry theEntry;
                    while ((theEntry = s.GetNextEntry()) != null)
                    {
                        string fileName = Path.GetFileName(theEntry.Name);
                        if (fileName != String.Empty)
                        {
                            using (FileStream streamWriter = File.Create(Config.INIsPath + Path.GetFileName(theEntry.Name)))
                            {
                                int size = 1024;
                                byte[] data = new byte[1024];
                                while (true)
                                {
                                    size = s.Read(data, 0, data.Length);
                                    if (size > 0)
                                    {
                                        streamWriter.Write(data, 0, size);
                                    }
                                    else
                                    {
                                        break;
                                    }
                                }
                            }
                        }
                    }
                }
            }
            /// Once all the files are extracted, we create a profile.dat file with the name of the current profile, just for 
            /// GUI purposes.
            StreamWriter sw = new StreamWriter(Config.INIsPath + "profile.dat"); sw.WriteLine(name); sw.Close();
            if (!silent) { MessageBox.Show("Profile activated", "Success!"); }
        }

        /// <summary>
        /// Deletes a profile (ZIP file). By default, it shows messages on success/error.
        /// </summary>
        /// <param name="name">Profile name to delete</param>
        public static void Delete(string name)
        {
            Delete(name, false);
        }

        /// <summary>
        /// Same as above, but with a flag to show messages on success/error or not.
        /// </summary>
        /// <param name="name"></param>
        /// <param name="silent"></param>
        public static void Delete(string name, bool silent)
        {
            try
            {
                File.Delete(Config.ProfilesPath + name + ".zip");
            }
            catch (Exception ex)
            {
                if (!silent) { MessageBox.Show(ex.Message, "Error"); }
            }
            finally
            {
                if (!silent) { MessageBox.Show("Profile deleted","Success!"); }
            }
        }
    }
    
    /// <summary>
    /// PPM stands for PCSX2 Profile Manager. This is more of a wrapper and utility class
    /// than anything else.
    /// </summary>
    public static class PPM
    {

        /// <summary>
        /// Get the current profile name by reading profile.dat (if it exists).
        /// </summary>
        /// <returns>Profile name if found, else, it returns a default "not found / default profile" text.</returns>
        public static string GetCurrentProfileName()
        {
            if (Directory.Exists(Config.INIsPath))
            {
                if (File.Exists(Config.INIsPath + "profile.dat"))
                {
                    using (StreamReader r = new StreamReader(Config.INIsPath + "profile.dat"))
                    {
                        return r.ReadLine();
                    }
                }
            }
            return "No profile detected / Default PCSX2 profile";
        }

        /// <summary>
        /// Same as above, but if the flag forProfile is set, it will return a short name to be used
        /// in ZIP file creation.
        /// </summary>
        /// <param name="forProfile">Return a short name for ZIP file creation.</param>
        /// <returns>Profile name if found, else, it returns a short name for ZIP file creation.</returns>
        public static string GetCurrentProfileName(bool forProfile)
        {
            if (forProfile == true)
            {
                if (Directory.Exists(Config.INIsPath))
                {
                    if (File.Exists(Config.INIsPath + "profile.dat"))
                    {
                        using (StreamReader r = new StreamReader(Config.INIsPath + "profile.dat"))
                        {
                            return r.ReadLine();
                        }
                    }
                }
                return "Default";
            }
            else { return GetCurrentProfileName(); }
        }

        /// <summary>
        /// Creates a new profile from the current configuration.
        /// </summary>
        public static void NewProfileFromCurrent()
        {
            /// Since there's nothing to configure already, we show all of the INI files
            /// to the user so it chooses which files to use as the new profile.
            frmSelectINIs dlg = new frmSelectINIs();
            DialogResult r = dlg.ShowDialog();
            if (r == DialogResult.OK)
            {
                /// If the user clicked OK in the frmSelectINIs dialog, we ask for
                /// the new profile's name.
                frmInputText dlg2 = new frmInputText();
                DialogResult r2 = dlg2.ShowDialog();
                if (r2 == DialogResult.OK)
                {
                    /// Time to save the profile, with the chosen files and profile name,
                    /// and displaying messages on error/success.
                    Profile.Save(dlg.SelectedFiles.ToArray(), dlg2.UserText, false);
                }
            }
        }

        /// <summary>
        /// Same as above, but it won't ask for INI file choosing since we start up with a blank,
        /// default PCSX2 config.
        /// </summary>
        public static void NewProfileFromDefault()
        {
            frmInputText dlg2 = new frmInputText();
            DialogResult r2 = dlg2.ShowDialog();
            if (r2 == DialogResult.OK)
            {
                Profile.Save(dlg2.UserText, false);
            }
        }

        /// <summary>
        /// Wrapper function to activate a profile. Rarely used I think.
        /// </summary>
        /// <param name="name">Profile's name to activate</param>
        /// <param name="silent">Show messages on success/error?</param>
        public static void ActivateProfile(string name, bool silent)
        {
            Profile.Activate(name, silent);
        }
    }
}
