﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.IO;
using System.Diagnostics;

namespace PCSX2_Profile_Manager
{
    public partial class frmMain : Form
    {
        #region Events
        public frmMain()
        {
            InitializeComponent();
        }

        /// <summary>
        /// If we get closed, we run Application.Exit(), just to make sure, but maybe this isn't needed.
        /// </summary>
        private void frmMain_FormClosed(object sender, FormClosedEventArgs e)
        {
            Application.Exit();
        }

        private void frmMain_Load(object sender, EventArgs e)
        {
            /// First we set the Config structure's initial values, before loading it, just for avoiding runtime
            /// errors with uninitialized values.
            Config.PCSX2Path = String.Empty;
            Config.DCAction = 0;
            Config.Load();
            if (Config.PCSX2Path == String.Empty)
            {
                /// Whoops, there's no config file, we'll attempt to create a new one, and inform the user.
                MessageBox.Show("Config.dat does not exist in the executable's folder. The application will create a new configuration file.", "Warning", MessageBoxButtons.OK, MessageBoxIcon.Warning);
                MessageBox.Show("The application needs to be configured.", "Warning", MessageBoxButtons.OK, MessageBoxIcon.Warning);

                /// Show the config dialog...
                frmConfig Conf = new frmConfig();
                if (Conf.ShowDialog() != DialogResult.OK)
                {
                    /// If, and only IF the dialog was closed successfully, but the PCSX2 EXEcutable path is still empty (shouldn't happen!),
                    /// we throw an error notifying the user, and quit.
                    if (Config.PCSX2Path == String.Empty)
                    {
                        MessageBox.Show("You haven't configured the application correctly. The program will abort.", "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                        Application.Exit();
                    }
                    /// Else, everything's cool and we load our profiles 'n stuff.
                }
            }
            Config.Load();
            RefreshProfiles();
        }
        #endregion 
        #region Menu items
        /// <summary>
        /// Show setup dialog, duh.
        /// </summary>
        private void mSetup_Click(object sender, EventArgs e)
        {
            frmConfig dlg = new frmConfig();
            DialogResult r = dlg.ShowDialog();
            if (r == DialogResult.OK)
            {
                RefreshProfiles();
            }
        }

        /// <summary>
        /// Create a new profile from current PCSX2 configuration
        /// </summary>
        private void mNew_Click(object sender, EventArgs e)
        {
            PPM.NewProfileFromCurrent();
        }

        /// <summary>
        /// Exit!
        /// </summary>
        private void mExit_Click(object sender, EventArgs e)
        {
            this.Close();
        }

        /// <summary>
        /// Create a new profile from PCSX2's default settings. The procedure is very similar to the Edit button's code, except we
        /// backup the current profile before deleting all the files so PCSX2 starts up without any configuration, ensuring the user is
        /// going to create a brand new profile.
        /// </summary>
        private void mDefault_Click(object sender, EventArgs e)
        {
            string oldProfile = PPM.GetCurrentProfileName(true);
            Profile.Save(oldProfile, true);
            Profile.Clear();
            MessageBox.Show("PCSX2 is going to be run now and the profile manager will minimize. Once you have finished configuring it, close PCSX2 so the profile manager knows you're finished and save the profile afterwards", "Careful", MessageBoxButtons.OK, MessageBoxIcon.Information);
            this.WindowState = FormWindowState.Minimized;
            ProcessStartInfo ps = new ProcessStartInfo(Config.PCSX2Exe);
            ps.WorkingDirectory = Config.PCSX2Path;
            Process.Start(ps).WaitForExit();
            this.WindowState = FormWindowState.Normal;
            DialogResult r = MessageBox.Show("Is it okay to save the profile you made?", "Question", MessageBoxButtons.YesNo, MessageBoxIcon.Information);
            if (r == DialogResult.Yes)
            {
                PPM.NewProfileFromDefault();
            }
            Profile.Activate(oldProfile, true);
            RefreshProfiles();
        }

        /// <summary>
        /// It shows the About dialog, heh :P
        /// </summary>
        private void mAbout_Click(object sender, EventArgs e)
        {
            StringBuilder sb = new StringBuilder();
            sb.AppendLine("PCSX2 Profile Manager");
            sb.AppendLine("by DARKGuy");
            sb.AppendLine();
            sb.AppendLine("This program is distributed freely without any warranties. Source code");
            sb.AppendLine("is available on the web, or you can contact me at dark.guy.2008@gmail.com");
            sb.AppendLine();
            sb.AppendLine("Enjoy! ^_^");
            MessageBox.Show(sb.ToString(), "About", MessageBoxButtons.OK, MessageBoxIcon.Information);
        }
        #endregion
        #region Buttons
        /// <summary>
        /// Activate the selected profile.
        /// </summary>
        private void btnActivate_Click(object sender, EventArgs e)
        {
            if (lbProfiles.SelectedIndex > -1)
            {
                PPM.ActivateProfile(lbProfiles.SelectedItem.ToString(), false);
                RefreshProfiles();
            }
        }

        /// <summary>
        /// Save the current profile
        /// </summary>
        private void btnSave_Click(object sender, EventArgs e)
        {
            Profile.Save(lblCurrentProfile.Text, false);
            RefreshProfiles();
        }

        /// <summary>
        /// Deletes the selected profile
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnDelete_Click(object sender, EventArgs e)
        {
            if (lbProfiles.SelectedIndex > -1)
            {
                DialogResult r = MessageBox.Show("Are you sure you want to delete the selected profile?", "Question", MessageBoxButtons.YesNo, MessageBoxIcon.Stop);
                if (r == DialogResult.Yes)
                {
                    Profile.Delete(lbProfiles.SelectedItem.ToString());
                }
            }
            RefreshProfiles();
        }

        /// <summary>
        /// Edit the selected profile. The function may be a bit tricky to understand for beginners.
        /// </summary>
        private void btnEdit_Click(object sender, EventArgs e)
        {
            /// First, we store the current profile's name, and the name of the profile we're going to edit
            string oldProfile = PPM.GetCurrentProfileName(true);
            string editProfile = lbProfiles.SelectedItem.ToString();

            /// We save the current profile, and activate the one we're gonna edit.
            Profile.Save(oldProfile, true);
            Profile.Activate(editProfile, true);

            /// We must inform the user we're gonna run PCSX2 so the user edits the profile
            MessageBox.Show("PCSX2 is going to be run now and the profile manager will minimize. Once you have finished editing your profile, close PCSX2 so the profile manager knows you're finished and save the profile afterwards", "Careful", MessageBoxButtons.OK, MessageBoxIcon.Information);

            /// We minimize the main window and start PCSX2 up.
            this.WindowState = FormWindowState.Minimized;
            ProcessStartInfo ps = new ProcessStartInfo(Config.PCSX2Exe);
            ps.WorkingDirectory = Config.PCSX2Path;
            Process.Start(ps).WaitForExit();

            /// Then we get back to normal, asking the user if it's okay to save the changes made to the profile.
            this.WindowState = FormWindowState.Normal;
            DialogResult r = MessageBox.Show("Is it okay to save the profile you just edited?", "Question", MessageBoxButtons.YesNo, MessageBoxIcon.Information);
            if (r == DialogResult.Yes)
            {
                /// Yes? okay, we save the changes.
                Profile.Save(editProfile, false);
            }

            /// It doesn't matter if we choose to save the changes or not, we're gonna get back to our old profile and refresh the list,
            /// in case we chose to save the changes.
            Profile.Activate(oldProfile, true);
            RefreshProfiles();
        }
        #endregion
        #region ListBoxes
        /// <summary>
        /// This function runs whenever the user double-clicks an item in the listbox.
        /// </summary>
        private void lbProfiles_DoubleClick(object sender, EventArgs e)
        {
            switch (Config.DCAction)
            {
                /// Do nothing
                case 0:
                    break;
                
                /// Activate profile
                case 1:
                    btnActivate_Click(null, null);
                    break;

                /// Activate & run PCSX2
                case 2:
                    if (lbProfiles.SelectedIndex > -1)
                    {
                        PPM.ActivateProfile(lbProfiles.SelectedItem.ToString(), false);
                        RefreshProfiles();
                        ProcessStartInfo ps = new ProcessStartInfo(Config.PCSX2Exe);
                        ps.WorkingDirectory = Config.PCSX2Path;
                        Process.Start(ps);
                    }
                    break;
            }
        }
        #endregion
        #region Helper functions
        /// <summary>
        /// This function refreshes the profiles. It loops through all the files in the
        /// [PCSX2Path]\inis\profiles folder and adds them to the list, stripping the
        /// path and extension. It's just a quick 'n dirty way to keep it simple by using
        /// the ZIP filename as the profile name.
        /// </summary>
        private void RefreshProfiles()
        {
            lbProfiles.Items.Clear();
            if (Config.ProfilesPath != null)
            {
                try
                {
                    string[] files = Directory.GetFiles(Config.ProfilesPath, "*.zip");
                    foreach (string file in files)
                    {
                        lbProfiles.Items.Add(Path.GetFileNameWithoutExtension(file));
                    }
                }
                finally
                {
                    lblCurrentProfile.Text = PPM.GetCurrentProfileName();
                }
            }
        }
        #endregion
    }
}
